var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
const packageStore = window.__PACKAGE_STORE__ ?? (window.__PACKAGE_STORE__ = /* @__PURE__ */ new Map());
function createPackageManager() {
  const packageLoaders = [];
  const componentsRecord = {};
  const packagesRef = [];
  async function addPackages(packages) {
    for (const item of packages) {
      if (!item.package && !item.id)
        continue;
      const newId = item.package ?? item.id;
      const isExist = packagesRef.some((_) => {
        const itemId = _.package ?? _.id;
        return itemId === newId;
      });
      if (!isExist) {
        packagesRef.push(item);
        if (!packageStore.has(newId)) {
          const loader = packageLoaders.find((loader2) => loader2.active(item));
          if (!loader)
            continue;
          try {
            const result = await loader.load(item, manager);
            if (result)
              packageStore.set(newId, result);
          } catch (e) {
            throw e;
          }
        }
      }
    }
  }
  function getPackageInfo(packageName) {
    return packagesRef.find((p) => p.package === packageName);
  }
  function getLibraryByPackageName(packageName) {
    const packageInfo = getPackageInfo(packageName);
    if (packageInfo) {
      return packageStore.get(packageInfo.package ?? packageInfo.id);
    }
  }
  function setLibraryByPackageName(packageName, library) {
    packageStore.set(packageName, library);
  }
  function resolveComponentMaps(componentMaps) {
    for (const map of componentMaps) {
      if (map.devMode === "lowCode") {
        const packageInfo = packagesRef.find((_) => {
          return _.id === map.id;
        });
        if (packageInfo) {
          componentsRecord[map.componentName] = packageInfo;
        }
      } else {
        if (packageStore.has(map.package)) {
          const library = packageStore.get(map.package);
          const paths = map.exportName && map.subName ? map.subName.split(".") : [];
          const exportName = map.exportName ?? map.componentName;
          if (map.destructuring) {
            paths.unshift(exportName);
          }
          let result = library;
          for (const path of paths) {
            result = result[path] || result;
          }
          const recordName = map.componentName ?? map.exportName;
          componentsRecord[recordName] = result;
        }
      }
    }
  }
  function getComponentsNameRecord(componentMaps) {
    if (componentMaps) {
      resolveComponentMaps(componentMaps);
    }
    return { ...componentsRecord };
  }
  function getComponent(componentName) {
    return componentsRecord[componentName];
  }
  function registerComponentByName(componentName, Component) {
    componentsRecord[componentName] = Component;
  }
  const manager = {
    addPackages,
    getPackageInfo,
    getLibraryByPackageName,
    setLibraryByPackageName,
    addPackageLoader(loader) {
      if (!loader.name || !packageLoaders.some((_) => _.name === loader.name)) {
        packageLoaders.push(loader);
      }
    },
    resolveComponentMaps,
    getComponentsNameRecord,
    getComponent,
    registerComponentByName
  };
  return manager;
}
function nonSetterProxy(target) {
  return new Proxy(target, {
    get(target2, p, receiver) {
      return Reflect.get(target2, p, receiver);
    },
    set() {
      return false;
    },
    has(target2, p) {
      return Reflect.has(target2, p);
    }
  });
}
function createPluginManager(context) {
  const installedPlugins = [];
  let readyToInstallPlugins = [];
  const setupContext = nonSetterProxy(context);
  async function install(plugin) {
    var _a, _b;
    if (installedPlugins.some((p) => p.name === plugin.name))
      return;
    if ((_a = plugin.dependsOn) == null ? void 0 : _a.some((dep) => !installedPlugins.some((p) => p.name === dep))) {
      readyToInstallPlugins.push(plugin);
      return;
    }
    await plugin.setup(setupContext);
    installedPlugins.push(plugin);
    for (const item of readyToInstallPlugins) {
      if ((_b = item.dependsOn) == null ? void 0 : _b.every((dep) => installedPlugins.some((p) => p.name === dep))) {
        await item.setup(setupContext);
        installedPlugins.push(item);
      }
    }
    if (readyToInstallPlugins.length) {
      readyToInstallPlugins = readyToInstallPlugins.filter(
        (item) => installedPlugins.some((p) => p.name === item.name)
      );
    }
  }
  return {
    async add(plugin) {
      if (installedPlugins.find((item) => item.name === plugin.name)) {
        console.warn("该插件已安装");
        return;
      }
      await install(plugin);
    }
  };
}
function definePlugin(plugin) {
  return plugin;
}
var freeGlobal = typeof global == "object" && global && global.Object === Object && global;
var freeSelf = typeof self == "object" && self && self.Object === Object && self;
var root = freeGlobal || freeSelf || Function("return this")();
var Symbol$1 = root.Symbol;
var objectProto$a = Object.prototype;
var hasOwnProperty$8 = objectProto$a.hasOwnProperty;
var nativeObjectToString$1 = objectProto$a.toString;
var symToStringTag$1 = Symbol$1 ? Symbol$1.toStringTag : void 0;
function getRawTag(value) {
  var isOwn = hasOwnProperty$8.call(value, symToStringTag$1), tag = value[symToStringTag$1];
  try {
    value[symToStringTag$1] = void 0;
    var unmasked = true;
  } catch (e) {
  }
  var result = nativeObjectToString$1.call(value);
  if (unmasked) {
    if (isOwn) {
      value[symToStringTag$1] = tag;
    } else {
      delete value[symToStringTag$1];
    }
  }
  return result;
}
var objectProto$9 = Object.prototype;
var nativeObjectToString = objectProto$9.toString;
function objectToString(value) {
  return nativeObjectToString.call(value);
}
var nullTag = "[object Null]", undefinedTag = "[object Undefined]";
var symToStringTag = Symbol$1 ? Symbol$1.toStringTag : void 0;
function baseGetTag(value) {
  if (value == null) {
    return value === void 0 ? undefinedTag : nullTag;
  }
  return symToStringTag && symToStringTag in Object(value) ? getRawTag(value) : objectToString(value);
}
function isObjectLike(value) {
  return value != null && typeof value == "object";
}
var symbolTag = "[object Symbol]";
function isSymbol(value) {
  return typeof value == "symbol" || isObjectLike(value) && baseGetTag(value) == symbolTag;
}
function arrayMap(array, iteratee) {
  var index = -1, length = array == null ? 0 : array.length, result = Array(length);
  while (++index < length) {
    result[index] = iteratee(array[index], index, array);
  }
  return result;
}
var isArray = Array.isArray;
var INFINITY$1 = 1 / 0;
var symbolProto = Symbol$1 ? Symbol$1.prototype : void 0, symbolToString = symbolProto ? symbolProto.toString : void 0;
function baseToString(value) {
  if (typeof value == "string") {
    return value;
  }
  if (isArray(value)) {
    return arrayMap(value, baseToString) + "";
  }
  if (isSymbol(value)) {
    return symbolToString ? symbolToString.call(value) : "";
  }
  var result = value + "";
  return result == "0" && 1 / value == -INFINITY$1 ? "-0" : result;
}
function isObject(value) {
  var type = typeof value;
  return value != null && (type == "object" || type == "function");
}
var asyncTag = "[object AsyncFunction]", funcTag$1 = "[object Function]", genTag = "[object GeneratorFunction]", proxyTag = "[object Proxy]";
function isFunction(value) {
  if (!isObject(value)) {
    return false;
  }
  var tag = baseGetTag(value);
  return tag == funcTag$1 || tag == genTag || tag == asyncTag || tag == proxyTag;
}
var coreJsData = root["__core-js_shared__"];
var maskSrcKey = function() {
  var uid = /[^.]+$/.exec(coreJsData && coreJsData.keys && coreJsData.keys.IE_PROTO || "");
  return uid ? "Symbol(src)_1." + uid : "";
}();
function isMasked(func) {
  return !!maskSrcKey && maskSrcKey in func;
}
var funcProto$2 = Function.prototype;
var funcToString$2 = funcProto$2.toString;
function toSource(func) {
  if (func != null) {
    try {
      return funcToString$2.call(func);
    } catch (e) {
    }
    try {
      return func + "";
    } catch (e) {
    }
  }
  return "";
}
var reRegExpChar = /[\\^$.*+?()[\]{}|]/g;
var reIsHostCtor = /^\[object .+?Constructor\]$/;
var funcProto$1 = Function.prototype, objectProto$8 = Object.prototype;
var funcToString$1 = funcProto$1.toString;
var hasOwnProperty$7 = objectProto$8.hasOwnProperty;
var reIsNative = RegExp(
  "^" + funcToString$1.call(hasOwnProperty$7).replace(reRegExpChar, "\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g, "$1.*?") + "$"
);
function baseIsNative(value) {
  if (!isObject(value) || isMasked(value)) {
    return false;
  }
  var pattern = isFunction(value) ? reIsNative : reIsHostCtor;
  return pattern.test(toSource(value));
}
function getValue(object, key) {
  return object == null ? void 0 : object[key];
}
function getNative(object, key) {
  var value = getValue(object, key);
  return baseIsNative(value) ? value : void 0;
}
var WeakMap = getNative(root, "WeakMap");
var defineProperty = function() {
  try {
    var func = getNative(Object, "defineProperty");
    func({}, "", {});
    return func;
  } catch (e) {
  }
}();
var MAX_SAFE_INTEGER$1 = 9007199254740991;
var reIsUint = /^(?:0|[1-9]\d*)$/;
function isIndex(value, length) {
  var type = typeof value;
  length = length == null ? MAX_SAFE_INTEGER$1 : length;
  return !!length && (type == "number" || type != "symbol" && reIsUint.test(value)) && (value > -1 && value % 1 == 0 && value < length);
}
function baseAssignValue(object, key, value) {
  if (key == "__proto__" && defineProperty) {
    defineProperty(object, key, {
      "configurable": true,
      "enumerable": true,
      "value": value,
      "writable": true
    });
  } else {
    object[key] = value;
  }
}
function eq(value, other) {
  return value === other || value !== value && other !== other;
}
var objectProto$7 = Object.prototype;
var hasOwnProperty$6 = objectProto$7.hasOwnProperty;
function assignValue(object, key, value) {
  var objValue = object[key];
  if (!(hasOwnProperty$6.call(object, key) && eq(objValue, value)) || value === void 0 && !(key in object)) {
    baseAssignValue(object, key, value);
  }
}
var MAX_SAFE_INTEGER = 9007199254740991;
function isLength(value) {
  return typeof value == "number" && value > -1 && value % 1 == 0 && value <= MAX_SAFE_INTEGER;
}
function isArrayLike(value) {
  return value != null && isLength(value.length) && !isFunction(value);
}
var objectProto$6 = Object.prototype;
function isPrototype(value) {
  var Ctor = value && value.constructor, proto = typeof Ctor == "function" && Ctor.prototype || objectProto$6;
  return value === proto;
}
var argsTag$1 = "[object Arguments]";
function baseIsArguments(value) {
  return isObjectLike(value) && baseGetTag(value) == argsTag$1;
}
var objectProto$5 = Object.prototype;
var hasOwnProperty$5 = objectProto$5.hasOwnProperty;
var propertyIsEnumerable = objectProto$5.propertyIsEnumerable;
var isArguments = baseIsArguments(/* @__PURE__ */ function() {
  return arguments;
}()) ? baseIsArguments : function(value) {
  return isObjectLike(value) && hasOwnProperty$5.call(value, "callee") && !propertyIsEnumerable.call(value, "callee");
};
const isArguments$1 = isArguments;
function stubFalse() {
  return false;
}
var freeExports$1 = typeof exports == "object" && exports && !exports.nodeType && exports;
var freeModule$1 = freeExports$1 && typeof module == "object" && module && !module.nodeType && module;
var moduleExports$1 = freeModule$1 && freeModule$1.exports === freeExports$1;
var Buffer2 = moduleExports$1 ? root.Buffer : void 0;
var nativeIsBuffer = Buffer2 ? Buffer2.isBuffer : void 0;
var isBuffer = nativeIsBuffer || stubFalse;
const isBuffer$1 = isBuffer;
var argsTag = "[object Arguments]", arrayTag = "[object Array]", boolTag = "[object Boolean]", dateTag = "[object Date]", errorTag = "[object Error]", funcTag = "[object Function]", mapTag$2 = "[object Map]", numberTag = "[object Number]", objectTag$2 = "[object Object]", regexpTag = "[object RegExp]", setTag$2 = "[object Set]", stringTag = "[object String]", weakMapTag$1 = "[object WeakMap]";
var arrayBufferTag = "[object ArrayBuffer]", dataViewTag$1 = "[object DataView]", float32Tag = "[object Float32Array]", float64Tag = "[object Float64Array]", int8Tag = "[object Int8Array]", int16Tag = "[object Int16Array]", int32Tag = "[object Int32Array]", uint8Tag = "[object Uint8Array]", uint8ClampedTag = "[object Uint8ClampedArray]", uint16Tag = "[object Uint16Array]", uint32Tag = "[object Uint32Array]";
var typedArrayTags = {};
typedArrayTags[float32Tag] = typedArrayTags[float64Tag] = typedArrayTags[int8Tag] = typedArrayTags[int16Tag] = typedArrayTags[int32Tag] = typedArrayTags[uint8Tag] = typedArrayTags[uint8ClampedTag] = typedArrayTags[uint16Tag] = typedArrayTags[uint32Tag] = true;
typedArrayTags[argsTag] = typedArrayTags[arrayTag] = typedArrayTags[arrayBufferTag] = typedArrayTags[boolTag] = typedArrayTags[dataViewTag$1] = typedArrayTags[dateTag] = typedArrayTags[errorTag] = typedArrayTags[funcTag] = typedArrayTags[mapTag$2] = typedArrayTags[numberTag] = typedArrayTags[objectTag$2] = typedArrayTags[regexpTag] = typedArrayTags[setTag$2] = typedArrayTags[stringTag] = typedArrayTags[weakMapTag$1] = false;
function baseIsTypedArray(value) {
  return isObjectLike(value) && isLength(value.length) && !!typedArrayTags[baseGetTag(value)];
}
function baseUnary(func) {
  return function(value) {
    return func(value);
  };
}
var freeExports = typeof exports == "object" && exports && !exports.nodeType && exports;
var freeModule = freeExports && typeof module == "object" && module && !module.nodeType && module;
var moduleExports = freeModule && freeModule.exports === freeExports;
var freeProcess = moduleExports && freeGlobal.process;
var nodeUtil = function() {
  try {
    var types = freeModule && freeModule.require && freeModule.require("util").types;
    if (types) {
      return types;
    }
    return freeProcess && freeProcess.binding && freeProcess.binding("util");
  } catch (e) {
  }
}();
var nodeIsTypedArray = nodeUtil && nodeUtil.isTypedArray;
var isTypedArray = nodeIsTypedArray ? baseUnary(nodeIsTypedArray) : baseIsTypedArray;
const isTypedArray$1 = isTypedArray;
function overArg(func, transform) {
  return function(arg) {
    return func(transform(arg));
  };
}
var nativeKeys = overArg(Object.keys, Object);
var objectProto$4 = Object.prototype;
var hasOwnProperty$4 = objectProto$4.hasOwnProperty;
function baseKeys(object) {
  if (!isPrototype(object)) {
    return nativeKeys(object);
  }
  var result = [];
  for (var key in Object(object)) {
    if (hasOwnProperty$4.call(object, key) && key != "constructor") {
      result.push(key);
    }
  }
  return result;
}
var reIsDeepProp = /\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/, reIsPlainProp = /^\w*$/;
function isKey(value, object) {
  if (isArray(value)) {
    return false;
  }
  var type = typeof value;
  if (type == "number" || type == "symbol" || type == "boolean" || value == null || isSymbol(value)) {
    return true;
  }
  return reIsPlainProp.test(value) || !reIsDeepProp.test(value) || object != null && value in Object(object);
}
var nativeCreate = getNative(Object, "create");
function hashClear() {
  this.__data__ = nativeCreate ? nativeCreate(null) : {};
  this.size = 0;
}
function hashDelete(key) {
  var result = this.has(key) && delete this.__data__[key];
  this.size -= result ? 1 : 0;
  return result;
}
var HASH_UNDEFINED$1 = "__lodash_hash_undefined__";
var objectProto$3 = Object.prototype;
var hasOwnProperty$3 = objectProto$3.hasOwnProperty;
function hashGet(key) {
  var data = this.__data__;
  if (nativeCreate) {
    var result = data[key];
    return result === HASH_UNDEFINED$1 ? void 0 : result;
  }
  return hasOwnProperty$3.call(data, key) ? data[key] : void 0;
}
var objectProto$2 = Object.prototype;
var hasOwnProperty$2 = objectProto$2.hasOwnProperty;
function hashHas(key) {
  var data = this.__data__;
  return nativeCreate ? data[key] !== void 0 : hasOwnProperty$2.call(data, key);
}
var HASH_UNDEFINED = "__lodash_hash_undefined__";
function hashSet(key, value) {
  var data = this.__data__;
  this.size += this.has(key) ? 0 : 1;
  data[key] = nativeCreate && value === void 0 ? HASH_UNDEFINED : value;
  return this;
}
function Hash(entries) {
  var index = -1, length = entries == null ? 0 : entries.length;
  this.clear();
  while (++index < length) {
    var entry = entries[index];
    this.set(entry[0], entry[1]);
  }
}
Hash.prototype.clear = hashClear;
Hash.prototype["delete"] = hashDelete;
Hash.prototype.get = hashGet;
Hash.prototype.has = hashHas;
Hash.prototype.set = hashSet;
function listCacheClear() {
  this.__data__ = [];
  this.size = 0;
}
function assocIndexOf(array, key) {
  var length = array.length;
  while (length--) {
    if (eq(array[length][0], key)) {
      return length;
    }
  }
  return -1;
}
var arrayProto = Array.prototype;
var splice = arrayProto.splice;
function listCacheDelete(key) {
  var data = this.__data__, index = assocIndexOf(data, key);
  if (index < 0) {
    return false;
  }
  var lastIndex = data.length - 1;
  if (index == lastIndex) {
    data.pop();
  } else {
    splice.call(data, index, 1);
  }
  --this.size;
  return true;
}
function listCacheGet(key) {
  var data = this.__data__, index = assocIndexOf(data, key);
  return index < 0 ? void 0 : data[index][1];
}
function listCacheHas(key) {
  return assocIndexOf(this.__data__, key) > -1;
}
function listCacheSet(key, value) {
  var data = this.__data__, index = assocIndexOf(data, key);
  if (index < 0) {
    ++this.size;
    data.push([key, value]);
  } else {
    data[index][1] = value;
  }
  return this;
}
function ListCache(entries) {
  var index = -1, length = entries == null ? 0 : entries.length;
  this.clear();
  while (++index < length) {
    var entry = entries[index];
    this.set(entry[0], entry[1]);
  }
}
ListCache.prototype.clear = listCacheClear;
ListCache.prototype["delete"] = listCacheDelete;
ListCache.prototype.get = listCacheGet;
ListCache.prototype.has = listCacheHas;
ListCache.prototype.set = listCacheSet;
var Map$1 = getNative(root, "Map");
function mapCacheClear() {
  this.size = 0;
  this.__data__ = {
    "hash": new Hash(),
    "map": new (Map$1 || ListCache)(),
    "string": new Hash()
  };
}
function isKeyable(value) {
  var type = typeof value;
  return type == "string" || type == "number" || type == "symbol" || type == "boolean" ? value !== "__proto__" : value === null;
}
function getMapData(map, key) {
  var data = map.__data__;
  return isKeyable(key) ? data[typeof key == "string" ? "string" : "hash"] : data.map;
}
function mapCacheDelete(key) {
  var result = getMapData(this, key)["delete"](key);
  this.size -= result ? 1 : 0;
  return result;
}
function mapCacheGet(key) {
  return getMapData(this, key).get(key);
}
function mapCacheHas(key) {
  return getMapData(this, key).has(key);
}
function mapCacheSet(key, value) {
  var data = getMapData(this, key), size = data.size;
  data.set(key, value);
  this.size += data.size == size ? 0 : 1;
  return this;
}
function MapCache(entries) {
  var index = -1, length = entries == null ? 0 : entries.length;
  this.clear();
  while (++index < length) {
    var entry = entries[index];
    this.set(entry[0], entry[1]);
  }
}
MapCache.prototype.clear = mapCacheClear;
MapCache.prototype["delete"] = mapCacheDelete;
MapCache.prototype.get = mapCacheGet;
MapCache.prototype.has = mapCacheHas;
MapCache.prototype.set = mapCacheSet;
var FUNC_ERROR_TEXT = "Expected a function";
function memoize(func, resolver) {
  if (typeof func != "function" || resolver != null && typeof resolver != "function") {
    throw new TypeError(FUNC_ERROR_TEXT);
  }
  var memoized = function() {
    var args = arguments, key = resolver ? resolver.apply(this, args) : args[0], cache = memoized.cache;
    if (cache.has(key)) {
      return cache.get(key);
    }
    var result = func.apply(this, args);
    memoized.cache = cache.set(key, result) || cache;
    return result;
  };
  memoized.cache = new (memoize.Cache || MapCache)();
  return memoized;
}
memoize.Cache = MapCache;
var MAX_MEMOIZE_SIZE = 500;
function memoizeCapped(func) {
  var result = memoize(func, function(key) {
    if (cache.size === MAX_MEMOIZE_SIZE) {
      cache.clear();
    }
    return key;
  });
  var cache = result.cache;
  return result;
}
var rePropName = /[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g;
var reEscapeChar = /\\(\\)?/g;
var stringToPath = memoizeCapped(function(string) {
  var result = [];
  if (string.charCodeAt(0) === 46) {
    result.push("");
  }
  string.replace(rePropName, function(match, number, quote, subString) {
    result.push(quote ? subString.replace(reEscapeChar, "$1") : number || match);
  });
  return result;
});
function toString(value) {
  return value == null ? "" : baseToString(value);
}
function castPath(value, object) {
  if (isArray(value)) {
    return value;
  }
  return isKey(value, object) ? [value] : stringToPath(toString(value));
}
var INFINITY = 1 / 0;
function toKey(value) {
  if (typeof value == "string" || isSymbol(value)) {
    return value;
  }
  var result = value + "";
  return result == "0" && 1 / value == -INFINITY ? "-0" : result;
}
function baseGet(object, path) {
  path = castPath(path, object);
  var index = 0, length = path.length;
  while (object != null && index < length) {
    object = object[toKey(path[index++])];
  }
  return index && index == length ? object : void 0;
}
function get(object, path, defaultValue) {
  var result = object == null ? void 0 : baseGet(object, path);
  return result === void 0 ? defaultValue : result;
}
var getPrototype = overArg(Object.getPrototypeOf, Object);
var objectTag$1 = "[object Object]";
var funcProto = Function.prototype, objectProto$1 = Object.prototype;
var funcToString = funcProto.toString;
var hasOwnProperty$1 = objectProto$1.hasOwnProperty;
var objectCtorString = funcToString.call(Object);
function isPlainObject(value) {
  if (!isObjectLike(value) || baseGetTag(value) != objectTag$1) {
    return false;
  }
  var proto = getPrototype(value);
  if (proto === null) {
    return true;
  }
  var Ctor = hasOwnProperty$1.call(proto, "constructor") && proto.constructor;
  return typeof Ctor == "function" && Ctor instanceof Ctor && funcToString.call(Ctor) == objectCtorString;
}
var DataView = getNative(root, "DataView");
var Promise$1 = getNative(root, "Promise");
var Set = getNative(root, "Set");
var mapTag$1 = "[object Map]", objectTag = "[object Object]", promiseTag = "[object Promise]", setTag$1 = "[object Set]", weakMapTag = "[object WeakMap]";
var dataViewTag = "[object DataView]";
var dataViewCtorString = toSource(DataView), mapCtorString = toSource(Map$1), promiseCtorString = toSource(Promise$1), setCtorString = toSource(Set), weakMapCtorString = toSource(WeakMap);
var getTag = baseGetTag;
if (DataView && getTag(new DataView(new ArrayBuffer(1))) != dataViewTag || Map$1 && getTag(new Map$1()) != mapTag$1 || Promise$1 && getTag(Promise$1.resolve()) != promiseTag || Set && getTag(new Set()) != setTag$1 || WeakMap && getTag(new WeakMap()) != weakMapTag) {
  getTag = function(value) {
    var result = baseGetTag(value), Ctor = result == objectTag ? value.constructor : void 0, ctorString = Ctor ? toSource(Ctor) : "";
    if (ctorString) {
      switch (ctorString) {
        case dataViewCtorString:
          return dataViewTag;
        case mapCtorString:
          return mapTag$1;
        case promiseCtorString:
          return promiseTag;
        case setCtorString:
          return setTag$1;
        case weakMapCtorString:
          return weakMapTag;
      }
    }
    return result;
  };
}
const getTag$1 = getTag;
var mapTag = "[object Map]", setTag = "[object Set]";
var objectProto = Object.prototype;
var hasOwnProperty = objectProto.hasOwnProperty;
function isEmpty(value) {
  if (value == null) {
    return true;
  }
  if (isArrayLike(value) && (isArray(value) || typeof value == "string" || typeof value.splice == "function" || isBuffer$1(value) || isTypedArray$1(value) || isArguments$1(value))) {
    return !value.length;
  }
  var tag = getTag$1(value);
  if (tag == mapTag || tag == setTag) {
    return !value.size;
  }
  if (isPrototype(value)) {
    return !baseKeys(value).length;
  }
  for (var key in value) {
    if (hasOwnProperty.call(value, key)) {
      return false;
    }
  }
  return true;
}
function baseSet(object, path, value, customizer) {
  if (!isObject(object)) {
    return object;
  }
  path = castPath(path, object);
  var index = -1, length = path.length, lastIndex = length - 1, nested = object;
  while (nested != null && ++index < length) {
    var key = toKey(path[index]), newValue = value;
    if (key === "__proto__" || key === "constructor" || key === "prototype") {
      return object;
    }
    if (index != lastIndex) {
      var objValue = nested[key];
      newValue = customizer ? customizer(objValue, key, nested) : void 0;
      if (newValue === void 0) {
        newValue = isObject(objValue) ? objValue : isIndex(path[index + 1]) ? [] : {};
      }
    }
    assignValue(nested, key, newValue);
    nested = nested[key];
  }
  return object;
}
function set(object, path, value) {
  return object == null ? object : baseSet(object, path, value);
}
function isJSExpression(v) {
  return isPlainObject(v) && v.type === "JSExpression" && typeof v.value === "string";
}
function isJSFunction(v) {
  return isPlainObject(v) && v.type === "JSFunction" && typeof v.value === "string";
}
function isJSSlot(v) {
  return isPlainObject(v) && v.type === "JSSlot" && v.value;
}
function isI18nNode(v) {
  return isPlainObject(v) && v.type === "i18n" && typeof v.key === "string";
}
function isLowCodeComponentSchema(v) {
  return isPlainObject(v) && v.type === "lowCode" && v.schema;
}
function someValue(obj, predicate) {
  if (!isPlainObject(obj) || isEmpty(obj))
    return false;
  if (predicate(obj))
    return true;
  for (const val of Object.values(obj)) {
    if (someValue(val, predicate))
      return true;
  }
  return false;
}
function processValue(obj, predicate, processor) {
  const innerProcess = (target, paths) => {
    if (Array.isArray(target)) {
      return target.map((item, idx) => innerProcess(item, [...paths, idx]));
    }
    if (!isPlainObject(target) || isEmpty(target))
      return target;
    if (!someValue(target, predicate))
      return target;
    if (predicate(target)) {
      return processor(target, paths);
    } else {
      const result = {};
      for (const [key, value] of Object.entries(target)) {
        result[key] = innerProcess(value, [...paths, key]);
      }
      return result;
    }
  };
  return innerProcess(obj, []);
}
const SYMBOL_SIGN = "__code__scope";
function createCodeRuntime(scopeOrValue = {}) {
  let runtimeScope = scopeOrValue[Symbol.for(SYMBOL_SIGN)] ? scopeOrValue : createScope(scopeOrValue);
  function run(code) {
    if (!code)
      return void 0;
    try {
      return new Function(
        "scope",
        `"use strict";return (function(){return (${code})}).bind(scope)();`
      )(runtimeScope.value);
    } catch (err) {
      console.error("%c eval error", code, runtimeScope.value, err);
      return void 0;
    }
  }
  function createFnBoundScope(code) {
    const fn = run(code);
    if (typeof fn !== "function")
      return void 0;
    return fn.bind(runtimeScope.value);
  }
  function parseExprOrFn(value) {
    return processValue(
      value,
      (data) => {
        return isJSExpression(data) || isJSFunction(data);
      },
      (node) => {
        let v;
        if (node.type === "JSExpression") {
          v = run(node.value);
        } else if (node.type === "JSFunction") {
          v = createFnBoundScope(node.value);
        }
        if (typeof v === "undefined" && node.mock) {
          return node.mock;
        }
        return v;
      }
    );
  }
  return {
    run,
    createFnBoundScope,
    parseExprOrFn,
    bindingScope(nextScope) {
      runtimeScope = nextScope;
    },
    getScope() {
      return runtimeScope;
    }
  };
}
function createScope(initValue) {
  const innerScope = { value: initValue };
  const proxyValue = new Proxy(/* @__PURE__ */ Object.create(null), {
    set(target, p, newValue, receiver) {
      return Reflect.set(target, p, newValue, receiver);
    },
    get(target, p, receiver) {
      let valueTarget = innerScope;
      while (valueTarget) {
        if (Reflect.has(valueTarget.value, p)) {
          return Reflect.get(valueTarget.value, p, receiver);
        }
        valueTarget = valueTarget.__parent;
      }
      return Reflect.get(target, p, receiver);
    }
  });
  const scope = {
    get value() {
      return proxyValue;
    },
    inject(name, value, force = false) {
      if (innerScope.value[name] && !force) {
        return;
      }
      innerScope.value[name] = value;
    },
    setValue(value, replace = false) {
      if (replace) {
        innerScope.value = { ...value };
      } else {
        innerScope.value = Object.assign({}, innerScope.value, value);
      }
    },
    createSubScope(initValue2) {
      const childScope = createScope(initValue2);
      childScope.__raw.__parent = innerScope;
      return childScope;
    }
  };
  Object.defineProperty(scope, Symbol.for(SYMBOL_SIGN), { get: () => true });
  Object.defineProperty(scope, "__raw", { get: () => innerScope });
  return scope;
}
function createEvent() {
  let events = [];
  function add(fn) {
    events.push(fn);
    return () => {
      events = events.filter((e) => e !== fn);
    };
  }
  function remove(fn) {
    events = events.filter((f) => fn !== f);
  }
  function list() {
    return [...events];
  }
  return {
    add,
    remove,
    list,
    clear() {
      events.length = 0;
    }
  };
}
const defaultTask = { run: (fn) => fn() };
const _createTask = () => defaultTask;
const createTask = typeof console.createTask !== "undefined" ? console.createTask : _createTask;
function createHookStore() {
  const hooksMap = /* @__PURE__ */ new Map();
  function hook(name, fn) {
    if (!name || typeof fn !== "function") {
      return () => {
      };
    }
    let hooks = hooksMap.get(name);
    if (!hooks) {
      hooks = createEvent();
      hooksMap.set(name, hooks);
    }
    hooks.add(fn);
    return () => remove(name, fn);
  }
  function call(name, ...args) {
    var _a;
    const hooks = ((_a = hooksMap.get(name)) == null ? void 0 : _a.list()) ?? [];
    for (const hookFn of hooks) {
      hookFn.call(null, ...args);
    }
  }
  function callAsync(name, ...args) {
    var _a;
    const hooks = ((_a = hooksMap.get(name)) == null ? void 0 : _a.list()) ?? [];
    const task = createTask(name.toString());
    return hooks.reduce(
      (promise, hookFunction) => promise.then(() => task.run(() => hookFunction(...args))),
      Promise.resolve()
    );
  }
  function callParallel(name, ...args) {
    var _a;
    const hooks = ((_a = hooksMap.get(name)) == null ? void 0 : _a.list()) ?? [];
    const task = createTask(name.toString());
    return Promise.all(hooks.map((hook2) => task.run(() => hook2(...args))));
  }
  function remove(name, fn) {
    const hooks = hooksMap.get(name);
    if (!hooks)
      return;
    if (fn) {
      hooks.remove(fn);
      if (hooks.list().length === 0) {
        hooksMap.delete(name);
      }
    } else {
      hooksMap.delete(name);
    }
  }
  function clear(name) {
    if (name) {
      remove(name);
    } else {
      hooksMap.clear();
    }
  }
  function getHooks(name) {
    var _a;
    return (_a = hooksMap.get(name)) == null ? void 0 : _a.list();
  }
  return {
    hook,
    call,
    callAsync,
    callParallel,
    remove,
    clear,
    getHooks
  };
}
const boostsValue = {};
const proxyBoostsValue = nonSetterProxy(boostsValue);
const appBoosts = {
  hookStore: createHookStore(),
  get value() {
    return proxyBoostsValue;
  },
  add(name, value, force = false) {
    if (boostsValue[name] && !force)
      return;
    boostsValue[name] = value;
  },
  remove(name) {
    if (boostsValue[name]) {
      delete boostsValue[name];
    }
  }
};
class RuntimeError extends Error {
  constructor(type, message) {
    super(message);
    this.type = type;
    appBoosts.hookStore.call(`app:error`, this);
  }
}
function throwRuntimeError(errorType, message) {
  return new RuntimeError(errorType, message);
}
function createAppSchema(schema) {
  if (!schema.version.startsWith("1.")) {
    throwRuntimeError("core", "schema version must be 1.x.x");
  }
  const schemaRef = structuredClone(schema);
  return {
    getComponentsTrees() {
      return schemaRef.componentsTree;
    },
    addComponentsTree(tree) {
      addArrayItem(schemaRef.componentsTree, tree, "id");
    },
    removeComponentsTree(id) {
      removeArrayItem(schemaRef.componentsTree, "id", id);
    },
    getComponentsMaps() {
      return schemaRef.componentsMap;
    },
    addComponentsMap(componentsMap) {
      addArrayItem(schemaRef.componentsMap, componentsMap, "componentName");
    },
    removeComponentsMap(componentName) {
      removeArrayItem(schemaRef.componentsMap, "componentName", componentName);
    },
    getPageConfigs() {
      return schemaRef.pages ?? [];
    },
    addPageConfig(page) {
      schemaRef.pages ?? (schemaRef.pages = []);
      addArrayItem(schemaRef.pages, page, "id");
    },
    removePageConfig(id) {
      schemaRef.pages ?? (schemaRef.pages = []);
      removeArrayItem(schemaRef.pages, "id", id);
    },
    getByKey(key) {
      return schemaRef[key];
    },
    updateByKey(key, updater) {
      const value = schemaRef[key];
      schemaRef[key] = typeof updater === "function" ? updater(value) : updater;
    },
    find(predicate) {
      return predicate(schemaRef);
    },
    getByPath(path) {
      return get(schemaRef, path);
    },
    updateByPath(path, updater) {
      set(schemaRef, path, typeof updater === "function" ? updater(this.getByPath(path)) : updater);
    }
  };
}
function addArrayItem(target, item, comparison) {
  const idx = target.findIndex((_) => _[comparison] === item[comparison]);
  if (idx > -1) {
    target.splice(idx, 1, item);
  } else {
    target.push(item);
  }
}
function removeArrayItem(target, comparison, comparisonValue) {
  const idx = target.findIndex((item) => item[comparison] === comparisonValue);
  if (idx > -1)
    target.splice(idx, 1);
}
function createAppFunction(appCreator) {
  if (typeof appCreator !== "function") {
    throw Error("The first parameter must be a function.");
  }
  return async (options) => {
    var _a, _b;
    const { schema, appScopeValue } = options;
    const appSchema = createAppSchema(schema);
    const appConfig = {};
    const packageManager = createPackageManager();
    const appScope = createScope({
      ...appScopeValue,
      constants: schema.constants ?? {}
    });
    const appContext = {
      schema: appSchema,
      config: appConfig,
      appScope,
      packageManager,
      boosts: appBoosts
    };
    const { appBase, renderer } = await appCreator(appContext, options);
    if (!("mount" in appBase) || !("unmount" in appBase)) {
      throw Error("appBase 必须返回 mount 和 unmount 方法");
    }
    const pluginManager = createPluginManager({
      ...appContext,
      renderer
    });
    if ((_a = options.plugins) == null ? void 0 : _a.length) {
      await Promise.all(options.plugins.map((p) => pluginManager.add(p)));
    }
    if ((_b = options.packages) == null ? void 0 : _b.length) {
      await packageManager.addPackages(options.packages);
    }
    return Object.assign(
      {
        schema,
        config: appConfig,
        use: pluginManager.add,
        get boosts() {
          return appBoosts.value;
        }
      },
      appBase
    );
  };
}
let idStart = 2311;
function guid() {
  return idStart++;
}
class Widget {
  constructor(raw) {
    __publicField(this, "proxyElements", []);
    __publicField(this, "renderObject");
    this.raw = raw;
    this.init();
  }
  init() {
  }
  get key() {
    var _a;
    return ((_a = this.raw) == null ? void 0 : _a.id) ?? `${guid()}`;
  }
  mapRenderObject(mapper) {
    this.renderObject = mapper(this);
    return this;
  }
  addProxyELements(el) {
    this.proxyElements.unshift(el);
  }
  build(builder) {
    return builder(this.renderObject ? [...this.proxyElements, this.renderObject] : []);
  }
}
class TextWidget extends Widget {
  constructor() {
    super(...arguments);
    __publicField(this, "type", "string");
  }
  init() {
    if (isJSExpression(this.raw)) {
      this.type = "expression";
    } else if (isI18nNode(this.raw)) {
      this.type = "i18n";
    }
  }
}
class ComponentWidget extends Widget {
  constructor() {
    super(...arguments);
    __publicField(this, "_children", []);
    __publicField(this, "_propsValue", {});
  }
  init() {
    if (this.raw.props) {
      this._propsValue = this.raw.props;
    }
    if (this.raw.children) {
      this._children = this.raw.children.map((child) => createWidget(child));
    }
  }
  get componentName() {
    return this.raw.componentName;
  }
  get props() {
    return this._propsValue ?? {};
  }
  get condition() {
    return this.raw.condition !== false;
  }
  get loop() {
    return this.raw.loop;
  }
  get loopArgs() {
    return this.raw.loopArgs ?? ["item", "index"];
  }
  get children() {
    return this._children;
  }
}
function createWidget(data) {
  if (typeof data === "string" || isJSExpression(data) || isI18nNode(data)) {
    return new TextWidget(data);
  } else if (data.componentName) {
    return new ComponentWidget(data);
  }
  throw Error(`unknown node data: ${JSON.stringify(data)}`);
}
function createContainer(options) {
  const {
    componentsTree,
    supCodeScope,
    initScopeValue = {},
    stateCreator,
    dataSourceCreator
  } = options;
  validContainerSchema(componentsTree);
  const instancesMap = /* @__PURE__ */ new Map();
  const subScope = supCodeScope ? supCodeScope.createSubScope(initScopeValue) : createScope(initScopeValue);
  const codeRuntime = createCodeRuntime(subScope);
  const initalState = codeRuntime.parseExprOrFn(componentsTree.state ?? {});
  const initalProps = codeRuntime.parseExprOrFn(componentsTree.props ?? {});
  const stateApi = stateCreator(initalState);
  const dataSourceApi = dataSourceCreator(componentsTree.dataSource, stateApi);
  const instanceApiObject = Object.assign(
    {
      props: initalProps,
      $(ref) {
        const insArr = instancesMap.get(ref);
        if (!insArr)
          return void 0;
        return insArr[0];
      },
      $$(ref) {
        return instancesMap.get(ref) ?? [];
      }
    },
    stateApi,
    dataSourceApi
  );
  if (componentsTree.methods) {
    for (const [key, fn] of Object.entries(componentsTree.methods)) {
      const customMethod = codeRuntime.createFnBoundScope(fn.value);
      if (customMethod) {
        instanceApiObject[key] = customMethod;
      }
    }
  }
  const containerCodeScope = subScope.createSubScope(instanceApiObject);
  codeRuntime.bindingScope(containerCodeScope);
  function setInstanceByRef(ref, ins) {
    let insArr = instancesMap.get(ref);
    if (!insArr) {
      insArr = [];
      instancesMap.set(ref, insArr);
    }
    insArr.push(ins);
  }
  function removeInstanceByRef(ref, ins) {
    const insArr = instancesMap.get(ref);
    if (insArr) {
      if (ins) {
        const idx = insArr.indexOf(ins);
        if (idx > 0)
          insArr.splice(idx, 1);
      } else {
        instancesMap.delete(ref);
      }
    }
  }
  function triggerLifeCycle(lifeCycleName, ...args) {
    if (!componentsTree.lifeCycles || !Object.keys(componentsTree.lifeCycles).includes(lifeCycleName)) {
      return;
    }
    const lifeCycleSchema = componentsTree.lifeCycles[lifeCycleName];
    if (isJSFunction(lifeCycleSchema)) {
      const lifeCycleFn = codeRuntime.createFnBoundScope(lifeCycleSchema.value);
      if (lifeCycleFn) {
        lifeCycleFn.apply(containerCodeScope.value, args);
      }
    }
  }
  return {
    get codeRuntime() {
      return codeRuntime;
    },
    get instanceApiObject() {
      return containerCodeScope.value;
    },
    getCssText() {
      return componentsTree.css;
    },
    triggerLifeCycle,
    setInstance: setInstanceByRef,
    removeInstance: removeInstanceByRef,
    createWidgets() {
      if (!componentsTree.children)
        return [];
      return componentsTree.children.map((item) => createWidget(item));
    }
  };
}
const CONTAINTER_NAME = ["Page", "Block", "Component"];
function validContainerSchema(schema) {
  if (!CONTAINTER_NAME.includes(schema.componentName)) {
    throw Error("container schema not valid");
  }
}
function createComponentFunction(stateCreator, componentCreator) {
  return (componentOptions) => {
    const {
      supCodeScope,
      initScopeValue = {},
      dataSourceCreator,
      componentsTree
    } = componentOptions;
    const container = createContainer({
      supCodeScope,
      initScopeValue,
      stateCreator,
      dataSourceCreator,
      componentsTree
    });
    return componentCreator(container, componentOptions);
  };
}
export {
  ComponentWidget,
  TextWidget,
  Widget,
  createAppFunction,
  createCodeRuntime,
  createComponentFunction,
  createContainer,
  createEvent,
  createHookStore,
  createScope,
  createWidget,
  definePlugin,
  isI18nNode,
  isJSExpression,
  isJSFunction,
  isJSSlot,
  isLowCodeComponentSchema,
  processValue,
  someValue
};
